#!/usr/bin/env ruby -wKU
#
# Adds or subtracts time from subtitles timestamps
# http://rubylearning.com/blog/2009/09/24/rpcfn-shift-subtitle-1/
MYVERSION = "1.1"
AUTHOR = "arno"
DATE = "10.01.10"


## Required libraries
require "optparse"
require "time"


## Define global variables
@options = {}


## Functions
def self.parse_cmd(argv = ::ARGV)
  sname = File.basename($0)

  begin
    opt = OptionParser.new do |opts|
      opts.banner = "\nUsage: #{sname} -o [add|sub] -t [time] input_file.srt output_file.srt"
      opts.separator "\nOptions:"

      # List of options
      opts.on( "-o", "--operation ADD|SUB", [:add, :sub], ::String, "Add or subtract time" ) do |op|
        # if operator is "add" then assign + else assign -
        @options[:operation] = op == "add" ? :+ : :-
      end

      opts.on( "-t", "--time TIME", ::String, "Amount of time (ex: 2,650)" ) do |t|
        t.gsub!( /,/, '.')
        @options[:time] = ("%.3f" % t).to_f
      end

      opts.on("-v", "--[no-]verbose", "Run verbosely") do |v|
        @options[:verbose] = v
      end

      opts.on("-d", "--[no-]debug", "Displays debug informations") do |d|
        @options[:debug] = @options[:verbose] = d
      end

      # Default options: help and version
      opts.separator "\nCommun options:"
      opts.on_tail("-h", "--help", "Print this help") do
        puts
        puts opts
        puts
        exit 0
      end

      opts.on_tail("--version", "Show version") do
        puts
        puts "%-9s: %s" % ["name", sname]
        puts "%-9s: %s (%s)" % ["version", MYVERSION, DATE]
        puts "%-9s: %s" % ["author", AUTHOR]
        puts
        exit 0
      end
    end

    opt.parse! argv

    @options[:input_file] = ARGV.shift
    @options[:output_file] = ARGV.shift

    raise ::ArgumentError.new, "Missing input file" unless @options[:input_file]
    raise ::ArgumentError.new, "Missing output file" unless @options[:output_file]

  rescue ::StandardError => e
    puts
    puts "Err: #{e}"
    puts e.backtrace if @options[:debug]
    puts opt
    puts
    exit 0
  end

  p @options if @options[:verbose]
end

def self.format_time(time)
  usec = time.usec / 1000
  usec = sprintf("%.3d", usec)
  "#{time.strftime('%H:%M:%S')},#{usec}"
end

def self.modify_time(times)
  tt = []
  times.each do |t|
    tobj = Time.parse(t)
    tt << format_time( tobj.send( @options[:operation], @options[:time] ))
  end
  tt
end


## Main
parse_cmd()

# Create the new srt file
newsrt = File.new( @options[:output_file], "r+" )

# Read existing srt file
File.foreach( @options[:input_file] ) do |line|

  # (00:00:16,688) --> (00:00:19,608)
  if line =~ /(\d{2}:\d{2}:\d{2},\d{3}) --> (\d{2}:\d{2}:\d{2},\d{3})/
    # Get the modified times in an array
    mt = modify_time( [$1, $2] )
    newsrt.puts "#{mt[0]} --> #{mt[1]}"

    if @options[:verbose]
      puts "- #{$1} --> #{$2}" 
      puts "+ #{mt[0]} --> #{mt[1]}" if @options[:verbose]
      puts
    end

  else
    newsrt.puts line
  end
end
